@@ -58,7 +58,7 @@ options:
                         Add dot graph attributes. E.g.
                         "rankdir=TB"
 
-FindImports requires Python 3.6 or later.
+FindImports requires Python 3.10 or later.
 
 Notes:
 
@@ -86,6 +86,7 @@ Copyright (c) 2003--2025 Marius Gedminas <marius@gedmin.as> and contributors.
 
 import argparse
 import ast
+import copy
 import doctest
 import fnmatch
 import json
@@ -100,115 +101,13 @@ import zipfile
 from operator import attrgetter
 
 
-__version__ = '2.8.0'
+__version__ = '3.0.0'
 __author__ = 'Marius Gedminas <marius@gedmin.as>'
 __licence__ = 'MIT'
 __url__ = 'https://github.com/mgedmin/findimports'
 
 
-STDLIB_MODNAMES_SET = getattr(sys, 'stdlib_module_names', frozenset([
-    # taken from Python 3.10
-    "__future__", "_abc", "_aix_support",
-    "_ast", "_asyncio", "_bisect",
-    "_blake2", "_bootsubprocess", "_bz2",
-    "_codecs", "_codecs_cn", "_codecs_hk",
-    "_codecs_iso2022", "_codecs_jp", "_codecs_kr",
-    "_codecs_tw", "_collections", "_collections_abc",
-    "_compat_pickle", "_compression", "_contextvars",
-    "_crypt", "_csv", "_ctypes",
-    "_curses", "_curses_panel", "_datetime",
-    "_dbm", "_decimal", "_elementtree",
-    "_frozen_importlib", "_frozen_importlib_external", "_functools",
-    "_gdbm", "_hashlib", "_heapq",
-    "_imp", "_io", "_json",
-    "_locale", "_lsprof", "_lzma",
-    "_markupbase", "_md5", "_msi",
-    "_multibytecodec", "_multiprocessing", "_opcode",
-    "_operator", "_osx_support", "_overlapped",
-    "_pickle", "_posixshmem", "_posixsubprocess",
-    "_py_abc", "_pydecimal", "_pyio",
-    "_queue", "_random", "_sha1",
-    "_sha256", "_sha3", "_sha512",
-    "_signal", "_sitebuiltins", "_socket",
-    "_sqlite3", "_sre", "_ssl",
-    "_stat", "_statistics", "_string",
-    "_strptime", "_struct", "_symtable",
-    "_thread", "_threading_local", "_tkinter",
-    "_tracemalloc", "_uuid", "_warnings",
-    "_weakref", "_weakrefset", "_winapi",
-    "_zoneinfo", "abc", "aifc",
-    "antigravity", "argparse", "array",
-    "ast", "asynchat", "asyncio",
-    "asyncore", "atexit", "audioop",
-    "base64", "bdb", "binascii",
-    "binhex", "bisect", "builtins",
-    "bz2", "cProfile", "calendar",
-    "cgi", "cgitb", "chunk",
-    "cmath", "cmd", "code",
-    "codecs", "codeop", "collections",
-    "colorsys", "compileall", "concurrent",
-    "configparser", "contextlib", "contextvars",
-    "copy", "copyreg", "crypt",
-    "csv", "ctypes", "curses",
-    "dataclasses", "datetime", "dbm",
-    "decimal", "difflib", "dis",
-    "distutils", "doctest", "email",
-    "encodings", "ensurepip", "enum",
-    "errno", "faulthandler", "fcntl",
-    "filecmp", "fileinput", "fnmatch",
-    "fractions", "ftplib", "functools",
-    "gc", "genericpath", "getopt",
-    "getpass", "gettext", "glob",
-    "graphlib", "grp", "gzip",
-    "hashlib", "heapq", "hmac",
-    "html", "http", "idlelib",
-    "imaplib", "imghdr", "imp",
-    "importlib", "inspect", "io",
-    "ipaddress", "itertools", "json",
-    "keyword", "lib2to3", "linecache",
-    "locale", "logging", "lzma",
-    "mailbox", "mailcap", "marshal",
-    "math", "mimetypes", "mmap",
-    "modulefinder", "msilib", "msvcrt",
-    "multiprocessing", "netrc", "nis",
-    "nntplib", "nt", "ntpath",
-    "nturl2path", "numbers", "opcode",
-    "operator", "optparse", "os",
-    "ossaudiodev", "pathlib", "pdb",
-    "pickle", "pickletools", "pipes",
-    "pkgutil", "platform", "plistlib",
-    "poplib", "posix", "posixpath",
-    "pprint", "profile", "pstats",
-    "pty", "pwd", "py_compile",
-    "pyclbr", "pydoc", "pydoc_data",
-    "pyexpat", "queue", "quopri",
-    "random", "re", "readline",
-    "reprlib", "resource", "rlcompleter",
-    "runpy", "sched", "secrets",
-    "select", "selectors", "shelve",
-    "shlex", "shutil", "signal",
-    "site", "smtpd", "smtplib",
-    "sndhdr", "socket", "socketserver",
-    "spwd", "sqlite3", "sre_compile",
-    "sre_constants", "sre_parse", "ssl",
-    "stat", "statistics", "string",
-    "stringprep", "struct", "subprocess",
-    "sunau", "symtable", "sys",
-    "sysconfig", "syslog", "tabnanny",
-    "tarfile", "telnetlib", "tempfile",
-    "termios", "textwrap", "this",
-    "threading", "time", "timeit",
-    "tkinter", "token", "tokenize",
-    "trace", "traceback", "tracemalloc",
-    "tty", "turtle", "turtledemo",
-    "types", "typing", "unicodedata",
-    "unittest", "urllib", "uu",
-    "uuid", "venv", "warnings",
-    "wave", "weakref", "webbrowser",
-    "winreg", "winsound", "wsgiref",
-    "xdrlib", "xml", "xmlrpc",
-    "zipapp", "zipfile", "zipimport",
-]))
+STDLIB_MODNAMES_SET = sys.stdlib_module_names
 
 
 def adjust_lineno(filename, lineno, name):
@@ -667,6 +566,8 @@ class ModuleGraph(object):
             self.warn(filename, '%s: unknown file name extension', filename)
         filename = os.path.abspath(filename)
         elements = filename.split(os.path.sep)
+        if elements[-1] == '__init__':
+            elements.pop()
         modname = []
         while elements:
             modname.append(elements.pop())
@@ -773,7 +674,7 @@ class ModuleGraph(object):
         """Is ``dotted_name`` the name of a package?"""
         candidate = self.isModule(dotted_name + '.__init__', extrapath)
         if candidate:
-            candidate = candidate[:-len(".__init__")]
+            candidate = candidate.removesuffix(".__init__")
         return candidate
 
     def collapseName(self, dotted_name, level):
@@ -972,6 +873,47 @@ class ModuleGraph(object):
         graph.modules = components
         return graph
 
+    def transitiveClosure(self):
+        """Compute a transitive closure of the graph."""
+        reachable = set()
+
+        def visit(modname):
+            reachable.add(modname)
+            module = self.modules.get(modname)
+            if module:
+                for impname in module.imports:
+                    if impname not in reachable:
+                        reachable.add(impname)
+                        visit(impname)
+
+        mods_reachable_from = {}
+        for modname in self.modules:
+            visit(modname)
+            mods_reachable_from[modname] = reachable
+            reachable = set()
+
+        return mods_reachable_from
+
+    def transitiveReduction(self):
+        """Compute a transitive reduction of the graph."""
+        mods_reachable_from = self.transitiveClosure()
+        modules = {}
+        for module in self.listModules():
+            reduced = copy.copy(module)
+            reduced.imports = set()
+            remaining = list(module.imports)
+            while remaining:
+                impname = remaining.pop()
+                for other in list(reduced.imports) + remaining:
+                    if impname in mods_reachable_from.get(other, ()):
+                        break
+                else:
+                    reduced.imports.add(impname)
+            modules[reduced.modname] = reduced
+        graph = ModuleGraph()
+        graph.modules = modules
+        return graph
+
     def printImportedNames(self):
         """Produce a report of imported names."""
         for module in self.listModules():
@@ -1155,6 +1097,10 @@ def main(argv=None):
     options.add_argument('-c', '--collapse', action='store_true',
                          dest='collapse_cycles',
                          help='collapse dependency cycles')
+    options.add_argument('-t', '--tred', action='store_true',
+                         dest='tred',
+                         help="compute the transitive reduction, i.e."
+                              " remove edges that are implied by other edges")
     options.add_argument('-T', '--tests', action='store_true',
                          dest='collapse_tests',
                          help="collapse packages named 'tests' and 'ftests'"
@@ -1216,6 +1162,10 @@ def main(argv=None):
         g = g.collapseTests()
     if args.collapse_cycles:
         g = g.collapseCycles()
+
+    if args.tred:
+        g = g.transitiveReduction()
+
     if args.rmprefix is not None:
         g = g.removePrefixes(args.rmprefix)
     g.external_dependencies = not args.noext
